function snr = calculateSNR(hwfft,f,nsig,f_start)
% snr = calculateSNR(hwfft,f,nsig=1) Estimate the signal-to-noise ratio, 
% given the in-band bins of a (Hann-windowed) fft and 
% the location of the input signal (f>0).
% For nsig=1, the input tone is contained in hwfft(f:f+2); 
%  this range is appropriate for a Hann-windowed fft.
% Each increment in nsig adds a bin to either side.
% The SNR is expressed in dB.

if nargin<3
    nsig = 1;
elseif nargin<4
    f_start = 0;
end

signalBins = [f-nsig+1:f+nsig+1];
signalBins = signalBins(signalBins>0);
signalBins = signalBins(signalBins<=length(hwfft));
s = norm(hwfft(signalBins));		% *4/(N*sqrt(3)) for true rms value;
noiseBins = 1:length(hwfft);

for i = 15:-1:2,
    harmBins = [i*f-nsig+1:i*f+nsig+1];
    harmBins = harmBins(harmBins<=length(hwfft));
    noiseBins(harmBins) = [];
end
noiseBins(signalBins) = [];
if f_start,
    noiseBins(1:f_start) = [];
end
n = norm(hwfft(noiseBins));
if n==0
    snr = Inf;
else
    snr = dbv(s/n);
end
noise = hwfft(noiseBins);
